# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("councilmatic_core", "0006_bill_subject"),
    ]

    operations = [
        migrations.AddField(
            model_name="post",
            name="division_ocd_id",
            field=models.CharField(max_length=255, default=""),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="post",
            name="shape",
            field=models.TextField(blank=True),
        ),
    ]



from django.conf.urls import url, include
from django.views.generic import RedirectView

from django.contrib import admin

from django.conf import settings

from . import views
from . import feeds

admin.autodiscover()


urlpatterns = [
    url(r"^$", views.IndexView.as_view(), name="index"),
    url(r"^search/$", RedirectView.as_view(), name="search"),
    url(r"^about/$", views.AboutView.as_view(), name="about"),
    url(r"^committees/$", views.CommitteesView.as_view(), name="committees"),
    url(
        r"^council-members/$",
        views.CouncilMembersView.as_view(),
        name="council_members",
    ),
    url(
        r"^committee/(?P<slug>[^/]+)/$",
        views.CommitteeDetailView.as_view(),
        name="committee_detail",
    ),
    url(
        r"^committee/(?P<slug>[^/]+)/events/rss/$",
        feeds.CommitteeDetailEventsFeed(),
        name="committee_detail_events_feed",
    ),
    url(
        r"^committee/(?P<slug>[^/]+)/actions/rss/$",
        feeds.CommitteeDetailActionFeed(),
        name="committee_detail_action_feed",
    ),
    url(
        r"^committee/(?P<slug>[^/]+)/widget/$",
        views.CommitteeWidgetView.as_view(),
        name="committee_widget",
    ),
    url(
        r"^legislation/(?P<slug>[^/]+)/$",
        views.BillDetailView.as_view(),
        name="bill_detail",
    ),
    url(
        r"^legislation/(?P<slug>[^/]+)/rss/$",
        feeds.BillDetailActionFeed(),
        name="bill_detail_action_feed",
    ),
    url(
        r"^legislation/(?P<slug>[^/]+)/widget/$",
        views.BillWidgetView.as_view(),
        name="bill_widget",
    ),
    url(r"^person/(?P<slug>[^/]+)/$", views.PersonDetailView.as_view(), name="person"),
    url(r"^person/(?P<slug>[^/]+)/rss/$", feeds.PersonDetailFeed(), name="person_feed"),
    url(
        r"^person/(?P<slug>[^/]+)/widget/$",
        views.PersonWidgetView.as_view(),
        name="person_widget",
    ),
    url(r"^events/$", views.EventsView.as_view(), name="events"),
    url(r"^events/rss/$", feeds.EventsFeed(), name="events_feed"),
    url(r"^event/(?P<slug>.+)/$", views.EventDetailView.as_view(), name="event_detail"),
    url(r"^flush-cache/(.*)/$", views.flush, name="flush"),
    url(r"^pdfviewer/$", views.pdfviewer, name="pdfviewer"),
]

if settings.USING_NOTIFICATIONS:
    urlpatterns.extend(
        [
            url(r"", include("notifications.urls")),
        ]
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-09-12 20:01
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("councilmatic_core", "0031_eventdocument_updated_at"),
    ]

    operations = [
        migrations.CreateModel(
            name="Subject",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("subject", models.CharField(max_length=255)),
            ],
        ),
        migrations.RemoveField(
            model_name="bill",
            name="subject",
        ),
        migrations.AlterField(
            model_name="eventdocument",
            name="updated_at",
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="subject",
            name="bill",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="subjects",
                to="councilmatic_core.Bill",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("councilmatic_core", "0007_auto_20151223_1150"),
    ]

    operations = [
        migrations.AddField(
            model_name="bill",
            name="ocr_full_text",
            field=models.TextField(blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-11-27 22:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("councilmatic_core", "0033_relatedbill"),
    ]

    operations = [
        migrations.AddField(
            model_name="event",
            name="guid",
            field=models.CharField(
                default=None, max_length=255, null=True, unique=True
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.utils.timezone import utc
import datetime


class Migration(migrations.Migration):

    dependencies = [
        ("councilmatic_core", "0009_bill_updated_at"),
    ]

    operations = [
        migrations.AddField(
            model_name="action",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 46, 57, 767919, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="actionrelatedentity",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 4, 415901, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="agendaitembill",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 8, 919814, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="billdocument",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 14, 71987, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="document",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 18, 455922, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="event",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 23, 135929, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="eventagendaitem",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 27, 383947, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="eventdocument",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 32, 151960, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="eventparticipant",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 36, 336258, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="legislativesession",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 42, 400193, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="membership",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 46, 375945, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="organization",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 50, 855978, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="person",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 47, 54, 823989, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="post",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 48, 1, 399994, tzinfo=utc),
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="sponsorship",
            name="updated_at",
            field=models.DateTimeField(
                auto_now=True,
                default=datetime.datetime(2016, 1, 20, 18, 48, 5, 392000, tzinfo=utc),
            ),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-10-17 17:01
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("councilmatic_core", "0023_auto_20161014_1648"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="agendaitembill",
            name="agenda_item",
        ),
        migrations.RemoveField(
            model_name="agendaitembill",
            name="bill",
        ),
        migrations.AddField(
            model_name="eventagendaitem",
            name="bill",
            field=models.ForeignKey(
                default="",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="related_agenda_items",
                to="councilmatic_core.Bill",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="eventagendaitem",
            name="note",
            field=models.CharField(default="", max_length=255),
            preserve_default=False,
        ),
        migrations.DeleteModel(
            name="AgendaItemBill",
        ),
    ]



import urllib

from haystack.query import SearchQuerySet

from django.contrib.syndication.views import Feed
from django.utils.feedgenerator import Rss201rev2Feed
from django.urls import reverse, reverse_lazy
from django.conf import settings

from .models import Person, Bill, Organization, Event
from .utils import to_datetime


class CouncilmaticFacetedSearchFeed(Feed):
    title_template = "feeds/search_item_title.html"
    description_template = "feeds/search_item_description.html"
    bill_model = Bill

    all_results = None
    sqs = (
        SearchQuerySet()
        .facet("bill_type")
        .facet("sponsorships", sort="index")
        .facet("controlling_body")
        .facet("inferred_status")
    )
    query = None

    def url_with_querystring(self, path, **kwargs):
        return path + "?" + urllib.parse.urlencode(kwargs)

    def get_object(self, request):
        self.queryDict = request.GET

        all_results = SearchQuerySet().all()
        facets = None

        if "selected_facets" in request.GET:
            facets = request.GET.getlist("selected_facets")

        if "q" in request.GET:
            self.query = request.GET["q"]
            results = all_results.filter(content=self.query)

            if facets:
                for facet in facets:
                    (facet_name, facet_value) = facet.split(":")
                    facet_name = facet_name.rsplit("_exact")[0]
                    results = results.narrow("%s:%s" % (facet_name, facet_value))
        elif facets:
            for facet in facets:
                (facet_name, facet_value) = facet.split(":")
                facet_name = facet_name.rsplit("_exact")[0]
                results = all_results.narrow("%s:%s" % (facet_name, facet_value))

        return results.order_by("-last_action_date")

    def title(self, obj):
        if self.query:
            title = (
                settings.SITE_META["site_name"]
                + ": Search for '"
                + self.query.capitalize()
                + "'"
            )
            # XXX: create a nice title based on all search parameters
        else:
            title = settings.SITE_META["site_name"] + ": Filtered Search"

        return title

    def link(self, obj):
        # return the main non-RSS search URL somehow
        # XXX maybe "quargs" - evz
        # return reverse('councilmatic_search', args=(searchqueryset=self.sqs,))
        url = self.url_with_querystring(
            reverse("{}:councilmatic_search_feed".format(settings.APP_NAME)),
            q=self.query,
        )
        return url

    def item_link(self, bill):
        return reverse("bill_detail", args=(bill.slug,))

    def item_pubdate(self, bill):
        return to_datetime(bill.last_action_date)

    def description(self, obj):
        return "Bills returned from search"

    def items(self, query):
        l_items = query[:20]
        pks = [i.pk for i in l_items]
        bills = self.bill_model.objects.filter(pk__in=pks).order_by("-last_action_date")
        return bills


class PersonDetailFeed(Feed):
    """The PersonDetailFeed provides an RSS feed for a given committee member,
    returning the most recent 20 bills for which they are the primary sponsor;
    and for each bill, the list of sponsores and the action history.
    """

    title_template = "feeds/person_detail_item_title.html"
    description_template = "feeds/person_detail_item_description.html"
    feed_type = Rss201rev2Feed
    NUM_RECENT_BILLS = 20

    def get_object(self, request, slug):
        o = Person.objects.get(slug=slug)
        return o

    def title(self, obj):
        return (
            settings.SITE_META["site_name"]
            + ": "
            + settings.CITY_VOCAB["COUNCIL_MEMBER"]
            + " %s: Recently Sponsored Bills" % obj.name
        )

    def link(self, obj):
        return reverse("person", args=(obj.slug,))

    def item_link(self, bill):
        # return the Councilmatic URL for the bill
        return reverse("bill_detail", args=(bill.slug,))

    def item_pubdate(self, bill):
        return to_datetime(bill.last_action_date)

    def description(self, obj):
        return "Recent sponsored bills from " + obj.name + "."

    def items(self, person):
        sponsored_bills = [s.bill for s in person.primary_sponsorships][:10]
        recent_sponsored_bills = sponsored_bills[: self.NUM_RECENT_BILLS]
        return recent_sponsored_bills


class CommitteeDetailEventsFeed(Feed):
    """The CommitteeDetailEventsFeed provides an RSS feed for a given committee,
    returning the most recent 20 events.
    """

    title_template = "feeds/committee_events_item_title.html"
    description_template = "feeds/committee_events_item_description.html"
    feed_type = Rss201rev2Feed
    NUM_RECENT_COMMITTEE_EVENTS = 20

    def get_object(self, request, slug):
        o = Organization.objects.get(slug=slug)
        return o

    def title(self, obj):
        return settings.SITE_META["site_name"] + ": " + obj.name + ": Recent Events"

    def link(self, obj):
        # return the Councilmatic URL for the committee
        return reverse("committee_detail", args=(obj.slug,))

    def item_link(self, event):
        # return the Councilmatic URL for the event
        return reverse("event_detail", args=(event.slug,))

    def item_pubdate(self, event):
        return event.start_time

    def description(self, obj):
        return "Events for committee %s" % obj.name

    def items(self, obj):
        return obj.recent_events.all()[: self.NUM_RECENT_COMMITTEE_EVENTS]


class CommitteeDetailActionFeed(Feed):
    """The CommitteeDetailActionFeed provides an RSS feed for a given committee,
    returning the most recent 20 actions on legislation.
    """

    # instead of defining item_title() or item_description(), use templates
    title_template = "feeds/committee_actions_item_title.html"
    description_template = "feeds/committee_actions_item_description.html"
    feed_type = Rss201rev2Feed
    NUM_RECENT_COMMITTEE_ACTIONS = 20

    def get_object(self, request, slug):
        o = Organization.objects.get(slug=slug)
        return o

    def title(self, obj):
        return settings.SITE_META["site_name"] + ": " + obj.name + ": Recent Actions"

    def link(self, obj):
        # return the Councilmatic URL for the committee
        return reverse("committee_detail", args=(obj.slug,))

    def item_link(self, action):
        # return the Councilmatic URL for the bill
        return reverse("bill_detail", args=(action.bill.slug,))

    def item_pubdate(self, action):
        return to_datetime(action.date_dt)

    def description(self, obj):
        return "Actions for committee %s" % obj.name

    def items(self, obj):
        return obj.recent_activity[: self.NUM_RECENT_COMMITTEE_ACTIONS]


class BillDetailActionFeed(Feed):
    """
    Return the last 20 actions for a given bill.
    """

    # instead of defining item_title() or item_description(), use templates
    title_template = "feeds/bill_actions_item_title.html"
    description_template = "feeds/bill_actions_item_description.html"
    feed_type = Rss201rev2Feed
    NUM_RECENT_BILL_ACTIONS = 20

    def get_object(self, request, slug):
        o = Bill.objects.get(slug=slug)
        return o

    def title(self, obj):
        return (
            settings.SITE_META["site_name"]
            + ": "
            + obj.friendly_name
            + ": Recent Actions"
        )

    def link(self, obj):
        # return the Councilmatic URL for the committee
        return reverse("bill_detail", args=(obj.slug,))

    def item_link(self, action):
        # Bill actions don't have their own pages, so just link to the Bill page (?)
        return reverse("bill_detail", args=(action.bill.slug,))

    def item_pubdate(self, action):
        return to_datetime(action.date_dt)

    def description(self, obj):
        return "Actions for bill %s" % obj.friendly_name

    def items(self, obj):
        return obj.ordered_actions[: self.NUM_RECENT_BILL_ACTIONS]


class EventsFeed(Feed):
    """
    Return the last 20 announced events as per, e.g.,
    https://nyc.councilmatic.org/events/
    """

    title_template = "feeds/events_item_title.html"
    description_template = "feeds/events_item_description.html"
    feed_type = Rss201rev2Feed
    NUM_RECENT_EVENTS = 20

    title = settings.CITY_COUNCIL_NAME + " " + "Recent Events"
    link = reverse_lazy("events")

    def item_link(self, event):
        # return the Councilmatic URL for the event
        return reverse("event_detail", args=(event.slug,))

    def item_pubdate(self, event):
        return event.start_time

    def description(self, obj):
        return "Recently Announced Events"

    def items(self, obj):
        return Event.objects.all()[: self.NUM_RECENT_EVENTS]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-10-17 21:40
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("councilmatic_core", "0024_auto_20161017_1201"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eventagendaitem",
            name="bill",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="related_agenda_items",
                to="councilmatic_core.Bill",
            ),
        ),
    ]



# Generated by Django 2.1.9 on 2019-06-18 17:30
import os

from django.conf import settings
import django.core.files.storage
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("councilmatic_core", "0046_subclass_ocd_billdocument"),
    ]

    operations = [
        migrations.AlterField(
            model_name="person",
            name="headshot",
            field=models.FileField(
                default="images/headshot_placeholder.png",
                storage=django.core.files.storage.FileSystemStorage(
                    base_url="/", location=os.path.join(settings.STATIC_ROOT)
                ),
                upload_to="images/headshots",
            ),
        ),
    ]
